import * as react from 'react';
import { UseQueryOptions, UseQueryResult } from '@tanstack/react-query';

type ServiceMethod<Request extends object = object, Response extends object = object> = (req: Request, initReq: RequestInitWithPathPrefix) => Promise<Response>;
interface RequestInitWithPathPrefix extends RequestInit {
    pathPrefix?: string;
}
type ServiceRequest<E extends ServiceMethod> = Parameters<E>[0];
type ServiceResponse<E extends ServiceMethod> = ReturnType<E>;
type ServiceError = Error | ErrorResponse;
type OnErrorHandler<E extends ServiceMethod> = (error: Error | ErrorResponse) => ReturnType<E> | null;
interface ErrorResponse {
    codeName?: string;
    code: number;
    message: string;
    details: unknown[];
}
type UseServiceQueryOptions<E extends ServiceMethod> = Omit<UseQueryOptions<Awaited<ServiceResponse<E>>, ServiceError>, 'queryFn'> & {
    onError?: OnErrorHandler<E>;
};
type UseServiceQueryResult<E extends ServiceMethod> = UseQueryResult<Awaited<ServiceResponse<E>>, ServiceError>;
declare function useServiceQuery<E extends ServiceMethod>(method: E, req: ServiceRequest<E>, options?: UseServiceQueryOptions<E>): UseServiceQueryResult<E>;
declare const ServiceContext: react.Context<RequestInitWithPathPrefix>;
declare function isErrorResponse(error: Error | ErrorResponse): error is ErrorResponse;

export { type ErrorResponse, type OnErrorHandler, type RequestInitWithPathPrefix, ServiceContext, type ServiceError, type ServiceMethod, type ServiceRequest, type ServiceResponse, isErrorResponse, useServiceQuery };
